@@ -9,8 +9,9 @@ import re
 from datetime import datetime
 from pathlib import Path
 
+import cchardet
 import click
-import markdown
+from bs4 import UnicodeDammit
 from ebooklib import epub
 
 from txt2ebook import __version__
@@ -41,6 +42,14 @@ logger = logging.getLogger(__name__)
     show_default=True,
     help="Set the author of the ebook.",
 )
+@click.option(
+    "--cover",
+    "-c",
+    type=click.Path(exists=True),
+    default=None,
+    show_default=True,
+    help="Set the cover of the ebook.",
+)
 @click.option(
     "--debug",
     "-d",
@@ -89,9 +98,10 @@ def main(ctx, **kwargs):
     """
     Console tool to convert txt file to different ebook format.
     """
-
     logging.basicConfig(
-        level=kwargs["debug"], format="[%(levelname)s] %(message)s"
+        level=kwargs["debug"] or logging.INFO,
+        format="[%(levelname).1s] %(asctime)s %(message)s",
+        datefmt="%Y-%m-%d %H:%M:%S",
     )
 
     ctx.obj = kwargs
@@ -100,27 +110,21 @@ def main(ctx, **kwargs):
         filename = Path(kwargs["filename"])
         title = kwargs["title"]
 
-        logger.debug("Processing txt file: '%s'.", filename.resolve())
+        logger.info("Processing txt file: '%s'.", filename.resolve())
 
-        with open(filename) as file:
-            content = file.read()
+        with open(filename, "rb") as file:
+            unicode = UnicodeDammit(file.read())
+            logger.info("Encoding detected: '%s'.", unicode.original_encoding)
+            content = unicode.unicode_markup
 
             if not content:
-                raise RuntimeError(f"Empty file content in '{filename}'.")
+                raise RuntimeError(f"Empty file content in '{filename}'!")
 
             content = tidy_content(content)
             chapters = parse_content(content)
 
-        if title:
-            output_filename = title
-        else:
-            match = re.search(r"《(.*)》", content)
-            if match:
-                title = match.group(1)
-                ctx.obj["title"] = title
-                output_filename = title
-            else:
-                output_filename = Path(filename).stem
+        ctx.obj["title"] = ctx.obj["title"] or find_book_title(content)
+        output_filename = ctx.obj["title"] or Path(filename).stem
 
         if not ctx.obj["author"]:
             match = re.search(r"作者：(.*)", content)
@@ -129,10 +133,28 @@ def main(ctx, **kwargs):
                 ctx.obj["author"] = author
 
         build_txt(filename, content, kwargs["no_backup"])
-        build_epub(output_filename, chapters)
+        if chapters:
+            build_epub(output_filename, chapters)
 
     except RuntimeError as error:
-        click.echo(f"Error: {str(error)}!", err=True)
+        click.echo(f"[E] {str(error)}!", err=True)
+
+
+def find_book_title(content):
+    """
+    Extract book title from the content of the txt file.
+    """
+    regex = r"书名：(.*)|【(.*)】|《(.*)》"
+    match = re.search(regex, content)
+    if match:
+        book_title = next(
+            (title for title in match.groups() if title is not None)
+        )
+        logger.info("Found book title: '%s'.", book_title)
+    else:
+        book_title = False
+        logger.info("No book title found from file!")
+    return book_title
 
 
 @click.pass_obj
@@ -150,7 +172,7 @@ def tidy_content(ctx, content):
     # Replace words/phrases based on regex.
     for search, replace in ctx["replace_regex"]:
         content = re.sub(
-            re.compile(rf"{search}", re.MULTILINE), replace, content
+            re.compile(rf"{search}", re.MULTILINE), rf"{replace}", content
         )
 
     # Delete whole line based on regex.
@@ -167,14 +189,14 @@ def tidy_content(ctx, content):
     # Remove incorrect closing quotation line wrapping.
     content = re.sub(r"\s+」", "」\n\n", content)
 
-    # Remove spaces or tabs at beginning of line.
-    content = re.sub(re.compile(r"^[ \t]+", re.MULTILINE), "", content)
+    # # Remove spaces or tabs at beginning of line.
+    # content = re.sub(re.compile(r"^[ \t]+", re.MULTILINE), "", content)
 
-    # Replace double spaces as single space.
-    content = content.replace("  ", " ")
+    # # Replace double spaces as single space.
+    # content = content.replace("  ", " ")
 
-    # Replace extra space in paragraph.
-    content = re.sub(re.compile(r"(?<![章卷])　", re.MULTILINE), "", content)
+    # # Replace extra space in paragraph.
+    # content = re.sub(re.compile(r"(?<![章卷])　", re.MULTILINE), "", content)
 
     # Append space to chapter name and chapter title.
     pattern = re.compile(
@@ -193,24 +215,36 @@ def tidy_content(ctx, content):
     for duplicate in duplicates:
         content = content.replace(duplicate[1], "", 1)
 
-    # Single spacing.
-    lines = content.split("\n")
-    content = "\n\n".join([line.strip() for line in lines if line])
+    # # Single spacing.
+    # lines = content.split("\n")
+    # content = "\n\n".join([line.strip() for line in lines if line])
 
     # Remove wrapping. Paragraph should be in one line.
     if ctx["remove_wrapping"]:
         unwrapped_content = ""
         for line in content.split("\n\n"):
-            if re.search(r"[…。？！]$", line) and "「" in line and "」" not in line:
+            # if a line contains more opening quote(「) than closing quote(」),
+            # we're still within the same paragraph.
+            # e.g.:
+            # 「...」「...
+            # 「...
+            if line.count("「") > line.count("」"):
                 unwrapped_content = unwrapped_content + line.strip()
             elif (
                 re.search(r"[…。？！]{1}」?$", line)
+                or re.search(r"」$", line)
                 or re.match(r"^[ \t]*……[ \t]*$", line)
                 or re.match(r"^「」$", line)
+                or re.match(r".*[》：＊\*]$", line)
+                or re.match(r".*[a-zA-Z0-9]$", line)
             ):
                 unwrapped_content = unwrapped_content + line.strip() + "\n\n"
             elif re.match(chapter_regexs(), line):
-                header = line.replace("\u3000\u3000", " ")
+                # replace full-width space with half-wdith space.
+                # looks nicer on the output.
+                header = line.replace("\u3000\u3000", " ").replace(
+                    "\u3000", " "
+                )
                 unwrapped_content = (
                     unwrapped_content + "\n\n" + header.strip() + "\n\n"
                 )
@@ -227,21 +261,31 @@ def parse_content(content):
     Parse the content into volumes (if exists) and chapters.
     """
 
-    volume_regex = "^[ \t]*第[0-9一二三四五六七八九十]*[卷册][^。~\n]*$"
+    volume_regex = "^[ \t]*第[0-9一二三四五六七八九十]*[集卷册][^。~\n]*$"
     volume_pattern = re.compile(rf"{volume_regex}", re.MULTILINE)
     volume_headers = re.findall(volume_pattern, content)
 
     if not volume_headers:
-        logger.debug("Found no volumes.")
+        logger.info("Parsed 0 volumes.")
         parsed_content = parse_chapters(content)
+        if parsed_content:
+            logger.info("Parsed %s chapters.", len(parsed_content))
+        else:
+            logger.error("Parsed 0 chapters.")
     else:
-        logger.debug("Found %s volumes.", len(volume_headers))
+        logger.info("Parsed %s volumes.", len(volume_headers))
         volume_bodies = re.split(volume_pattern, content)
         volumes = list(zip(volume_headers, volume_bodies[1:]))
 
         parsed_content = []
         for volume_header, body in volumes:
-            parsed_content.append((volume_header, parse_chapters(body)))
+            parsed_body = parse_chapters(body)
+            if parsed_body:
+                parsed_content.append((volume_header, parsed_body))
+            else:
+                logger.error(
+                    "Parsed 0 chapters for volume: '%s'.", volume_header
+                )
 
     return parsed_content
 
@@ -255,12 +299,12 @@ def parse_chapters(content):
     chapter_headers = re.findall(chapter_pattern, content)
 
     if not chapter_headers:
-        raise RuntimeError("No chapter headers found!")
+        return False
+    else:
+        bodies = re.split(chapter_pattern, content)
+        chapters = list(zip(chapter_headers, bodies[1:]))
 
-    bodies = re.split(chapter_pattern, content)
-    chapters = list(zip(chapter_headers, bodies[1:]))
-
-    return chapters
+        return chapters
 
 
 def chapter_regexs():
@@ -268,11 +312,12 @@ def chapter_regexs():
     Regex rules for chapter headers.
     """
     regexs = (
-        "^[ \t]*第[.0-9零一二三四五六七八九十百千两]*[章回节篇][^。~\n]*$",
-        "^[ \t]*楔子.*$",
-        "^[ \t]*序[章幕曲].*$",
+        "^[ \t]*第[.0-9零一二三四五六七八九十百千两]*[章篇折][^。~\n]*$",
+        "^[ \t]*[楔引]子.*$",
+        "^[ \t]*序[章幕曲]?.*$",
         "^[ \t]*前言.*$",
         "^[ \t]*[内容]*简介.*$",
+        "^[ \t]*[号番]外篇.*$",
         "^[ \t]*尾声$",
     )
     return "|".join(regexs)
@@ -292,11 +337,11 @@ def build_txt(filename, parsed_content, no_backup):
             )
         )
         os.rename(txt_filename, backup_filename)
-        logger.debug("Backup txt file: '%s'.", backup_filename)
+        logger.info("Backup txt file: '%s'.", backup_filename)
 
     with open(txt_filename, "w") as file:
         file.write(parsed_content)
-        logger.debug("Overwrite txt file: '%s'.", txt_filename.resolve())
+        logger.info("Overwrite txt file: '%s'.", txt_filename.resolve())
 
 
 @click.pass_obj
@@ -316,6 +361,10 @@ def build_epub(ctx, output_filename, parsed_content):
     if ctx["author"]:
         book.add_author(ctx["author"])
 
+    if ctx["cover"]:
+        book.set_cover("cover.jpg", open(ctx["cover"], "rb").read())
+        book.spine += ["cover"]
+
     book.add_item(epub.EpubNcx())
     book.add_item(epub.EpubNav())
 
@@ -338,25 +387,31 @@ def build_epub(ctx, output_filename, parsed_content):
             book.spine += [html_chapter]
             book.toc += [html_chapter]
 
-    logger.debug("Generating epub file: '%s'.", output_filename + ".epub")
+    logger.info("Generating epub file: '%s'.", output_filename + ".epub")
     epub.write_epub(output_filename + ".epub", book, {})
 
 
 def build_html_chapter(title, body, volume=None):
     """
-    Generates the whole chapter from Markdown to HTML.
+    Generates the whole chapter to HTML.
     """
     if volume:
         filename = f"{volume}_{title}"
+        logger.debug(f"  {title}")
     else:
         filename = title
+        logger.debug(title)
 
     filename = filename.replace(" ", "_")
 
-    logger.debug(title)
+    html = f"<h2>{title}</h2>"
+    for paragraph in body.split("\n\n"):
+        paragraph = paragraph.replace(" ", "").replace("\n", "")
+        html = html + f"<p>{paragraph}</p>"
+
     return epub.EpubHtml(
         title=title,
-        content=markdown.markdown("# " + title + "\n\n" + body),
+        content=html,
         file_name=filename + ".xhtml",
     )
 
